from typing import Dict, Text
from unittest.mock import Mock

from _pytest.monkeypatch import MonkeyPatch
from pytest_sanic.utils import TestClient
from rasax.community.services.data_service import DataService


async def test_create_entity_synonym_mapped_values(
    test_client: TestClient, auth_header: Dict[Text, Text], monkeypatch: MonkeyPatch
):
    mapped_values = ["one", "two", "three"]
    payload = [{"value": i} for i in mapped_values]
    synonym_id = 10
    project = "default"

    add_entity_synonym_mapped_values = Mock(return_value=payload)
    monkeypatch.setattr(
        DataService,
        "add_entity_synonym_mapped_values",
        add_entity_synonym_mapped_values,
    )

    response = await test_client.post(
        f"/api/projects/{project}/synonyms/{synonym_id}",
        json={"mapped_values": payload},
        headers=auth_header,
    )

    assert response.status == 201
    response_body = await response.json()
    assert response_body == payload
    add_entity_synonym_mapped_values.assert_called_once_with(
        project, synonym_id, mapped_values
    )


async def test_test_create_entity_synonym_mapped_values_error(
    test_client: TestClient, auth_header: Dict[Text, Text], monkeypatch: MonkeyPatch
):

    add_entity_synonym_mapped_values = Mock(side_effect=ValueError)
    monkeypatch.setattr(
        DataService,
        "add_entity_synonym_mapped_values",
        add_entity_synonym_mapped_values,
    )

    response = await test_client.post(
        f"/api/projects/default/synonyms/1",
        json={"mapped_values": {"value": "test"}},
        headers=auth_header,
    )

    assert response.status == 400


async def test_get_entity_synonyms_with_mapped_value_query(
    test_client: TestClient, auth_header: Dict[Text, Text], monkeypatch: MonkeyPatch
):
    matching_synonym = {"id": 123, "synonym_reference": "test"}
    get_synonym_by_mapped_value = Mock(return_value=matching_synonym)

    monkeypatch.setattr(
        DataService, "get_synonym_by_mapped_value", get_synonym_by_mapped_value
    )

    mapped_value = "I was mapped to a synonym"
    response = await test_client.get(
        f"/api/projects/default/synonyms?mapped_value={mapped_value}",
        headers=auth_header,
    )

    assert response.status == 200
    get_synonym_by_mapped_value.assert_called_once_with(mapped_value, "default")

    response_body = await response.json()

    assert response_body == [matching_synonym]


async def test_get_entity_synonyms(
    test_client: TestClient, auth_header: Dict[Text, Text], monkeypatch: MonkeyPatch
):
    get_entity_synonyms = Mock(return_value=[])

    monkeypatch.setattr(DataService, "get_entity_synonyms", get_entity_synonyms)

    response = await test_client.get(
        f"/api/projects/default/synonyms", headers=auth_header
    )

    assert response.status == 200
    get_entity_synonyms.assert_called_once_with("default", nlu_format=False)
    assert (await response.json()) == []
