from typing import Text, Dict, Optional

import pytest
from aioresponses import aioresponses
from pathlib import Path

import rasax.community.jwt
from rasax.community import utils
from rasax.community.constants import SHARE_YOUR_BOT_CHANNEL_NAME, DEFAULT_CHANNEL_NAME
from rasax.community.services.stack_service import StackService
from rasax.community.services.user_service import GUEST
from tests.unit.conftest import TEST_STACK_URL, generate_key_pair


@pytest.mark.parametrize(
    "fingerprint_payload, result", [({}, False), ({"some": "fingerprint"}, True)]
)
async def test_active_model(
    stack_service: StackService, fingerprint_payload: Dict[Text, Text], result: bool
):
    status_url = f"{TEST_STACK_URL}/status"
    with aioresponses() as mocked:
        mocked.get(
            status_url, payload={"model": "path", "fingerprint": fingerprint_payload}
        )
        has_active_model = await stack_service.has_active_model()
        assert has_active_model is result


@pytest.mark.parametrize(
    "user, expected_channel",
    [
        ({"user": {"roles": [GUEST], "username": "me"}}, SHARE_YOUR_BOT_CHANNEL_NAME),
        (
            {"user": {"roles": ["some role", "another role"], "username": "me"}},
            DEFAULT_CHANNEL_NAME,
        ),
    ],
)
def test_get_user_properties_from_bearer(
    tmp_path: Path, user: Dict, expected_channel: Optional[Text]
):
    private_key_file, public_key_file = generate_key_pair(tmp_path, 0)
    private_key = utils.file_as_bytes(private_key_file)
    public_key = utils.file_as_bytes(public_key_file)
    bearer_token = rasax.community.jwt.bearer_token(user, private_key)
    assert StackService._get_user_properties_from_bearer(bearer_token, public_key) == (
        expected_channel,
        user["user"]["username"],
    )
